#!/usr/bin/perl
use 5.014;
use strict;
use warnings FATAL => 'all';
use Test::More;

plan skip_all => 'for authors only -- define $ENV{AUTHOR_TESTING}' unless ( $ENV{AUTHOR_TESTING} ||  $ENV{RELEASE_TESTING} );

eval 'use Test::Version qw( version_all_ok ), { is_strict => 1, consistent => 1, }';
plan skip_all => 'Test::Version required for this test' if $@;
plan tests => 3;

version_all_ok( );
